from unittest import TestCase

from src.spreadsheets import Table


class TestTable(TestCase):

    EXAMPLE_TABLE_COLUMN_COUNT = 11
    EXAMPLE_TABLE_ROW_COUNT = 5

    def setUp(self):
        self.table = Table(
            headers=(
                'first column',
                'second column',
                'third column',
            ),
            rows=None,
        )

    def _table_from_file(self):
        import os
        filename = os.path.join(os.path.dirname(__file__), 'example.xlsx')
        return Table.from_xlsx(filename)

    def test_table_not_None(self):
        self.assertIsNotNone(self.table)

    def test_headers(self):
        self.assertEqual(self.table.headers, (
                'first column',
                'second column',
                'third column',
            )
        )

    def test_add_row(self):
        row = {
            'first column': 'first',
            'second column': 'second',
            'third column': 'third',
        }
        self.table.add(row)
        self.assertEqual(1, self.table.rows_count)

    def test_add_row_with_not_exists_column(self):
        row = {
            'first column': 'first',
            'second column': 'second',
            'third column not exists': 'third',
        }

        with self.assertRaises(Exception):
            self.table.add(row)

    def test_table_from_xlsx(self):
        table = self._table_from_file()

        self.assertIsNotNone(table)
        self.assertEqual(table.columns_count, self.EXAMPLE_TABLE_COLUMN_COUNT)
        self.assertEqual(table.rows_count, self.EXAMPLE_TABLE_ROW_COUNT)

    def test_merge_tables(self):
        table1 = self._table_from_file()
        table2 = self._table_from_file()

        table1.merge(table2)

        self.assertEqual(table1.columns_count, self.EXAMPLE_TABLE_COLUMN_COUNT)
        self.assertEqual(table1.rows_count, self.EXAMPLE_TABLE_ROW_COUNT*2)



import tkinter as tk
from tkinter import filedialog
from tkinter import messagebox

from .spreadsheets import merge_directory

class MainFrame(tk.Frame):
    def __init__(self, master):
        super().__init__(master)
        self.root = master

        self.directory_var = tk.StringVar()

        tk.Label(self, text="Source:").pack(side=tk.LEFT)

        tk.Entry(
            self,
            textvariable=self.directory_var,
        ).pack(side=tk.LEFT, expand=True)

        tk.Button(
            self,
            text='...',
            command=self.directory_choose_button,
        ).pack()

        tk.Button(
            self,
            text='close',
            command=self.close_button
        ).pack(side=tk.LEFT, expand=True)

        tk.Button(
            self,
            text='merge',
            command=self.merge_button
        ).pack(side=tk.LEFT, expand=True)

        self.pack()

    def directory_choose_button(self):
        directory = filedialog.askdirectory()
        print(directory)
        self.directory_var.set(directory)

    def close_button(self):
        self.root.destroy()

    def merge_button(self):
        merge_directory(
            self.directory_var.get(),
            'Сводная таблица.xlsx'
        )

        messagebox.showinfo(title='Merge', message='done')



def main():
    root = tk.Tk()
    frame = MainFrame(root)
    frame.mainloop()

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name='merge spreadsheets',
    version='0.0.1',
    packages=find_packages(),
    long_description='gui for merge spreadsheets',
    include_package_data=True,
    test_suite='tests',
    entry_points={
        'console_scripts':
            ['merge_spreadsheets = src.gui_merge_spreadsheets:main']
    },
    author='msBooM',
    author_email='work-msboom@yandex.ru',
)



import copy
from collections import OrderedDict

import openpyxl


class Table:

    def __init__(self, headers=None, rows=None):
        if rows is None:
            rows = []

        self.headers = headers
        self.rows = rows

    def _new_row(self):
        return [None] * len(self.headers)

    def add(self, row):
        new_row = self._new_row()

        for i, k in enumerate(row.keys()):
            if k in self.headers:
                new_row[i] = row[k]
            else:
                raise Exception("Column '{}' does not exists".format(k))

        self.rows.append(new_row)

    @property
    def rows_count(self):
        return len(self.rows)

    @property
    def columns_count(self):
        return len(self.headers)

    @classmethod
    def from_xlsx(cls, filename):
        wb = openpyxl.load_workbook(filename=filename)
        ws = wb.worksheets[0]

        headers = []
        rows = []

        c = 1
        while True:
            cell = ws.cell(row=1, column=c)

            if cell.value is None:
                break
            c = c + 1

            headers.append(cell.value)

        columns_count = len(headers)

        r = 2
        while True:
            row = [
                ws.cell(row=r, column=c).value for
                c in range(1, columns_count+1)
            ]

            if row.count(None) == columns_count:
                break

            rows.append(row)
            r = r + 1

        return cls(headers=tuple(headers), rows=rows)

    def merge(self, other):
        if set(self.headers) != set(other.headers):
            # todo: check duplicates column names
            raise Exception('Tables with differences columns can not be merged')
        pass

        row_template = OrderedDict.fromkeys(self.headers)
        for r in other.rows:
            row = copy.copy(row_template)

            for i, c in enumerate(other.headers):
                row[c] = r[i]

            # todo: check headers
            self.rows.append(row.values())

    def print(self):
        print('|'.join(self.headers))
        for r in self.rows:
            print('|'.join(str(i) for i in r))

    def save_to_file(self, filename, template=None):
        work_book = openpyxl.Workbook()
        sheet = work_book.worksheets[0]
        for i, h in enumerate(self.headers, start=1):
            sheet.cell(row=1, column=i).value = h

        for r, row in enumerate(self.rows, start=2):
            for c, value in enumerate(row, start=1):
                sheet.cell(row=r, column=c).value = value

        work_book.save(filename)


def merge_tables(tables):
    merged_table = Table(headers=tables[0].headers)

    for t in tables:
        merged_table.merge(t)

    return merged_table


def merge_files(files, output_filename):
    tables = [Table.from_xlsx(f) for f in files]
    merged_table = merge_tables(tables)

    merged_table.save_to_file(output_filename)

def merge_directory(directory, result_filename):
    import os
    files = (
        os.path.join(directory, f) for f in os.listdir(directory)
        if (
            f.endswith('.xlsx') and
            f != result_filename
        )
    )

    merge_files(files, os.path.join(directory, result_filename))

